using System;
using System.IO;
using System.Reflection;
using System.Runtime.Loader;

namespace WingCloudServer
{
    public class AssemblyLoader
    {
        //private AssemblyLoadContext _assemblyLoadContext;
        private AssemblyDependencyResolver _assemblyDependencyResolver;

        /// <summary>
        /// 加载依赖项
        /// </summary>
        /// <param name="assemblyName">程序集</param>
        /// <returns></returns>
        public Assembly LoadAssemblyDependencies(string assemblyPath)
        {
            _assemblyDependencyResolver = new AssemblyDependencyResolver(Path.GetDirectoryName(assemblyPath));
            AssemblyLoadContext.Default.Resolving += (assemblyLoadContext, assemblyName) =>
            {
                var path = _assemblyDependencyResolver.ResolveUnmanagedDllToPath(assemblyName.FullName);
                if(string.IsNullOrWhiteSpace(path))
                {
                    path = Path.Combine(Path.GetDirectoryName(assemblyPath), assemblyName.Name+".dll");
                }
                if (string.IsNullOrWhiteSpace(path) || !File.Exists(path))
                {
                    path = FindDependencyByInternalServiceDir(assemblyName.Name);
                }
                if (string.IsNullOrWhiteSpace(path) || !File.Exists(path))
                {
                    path = FindDependencyByNetServiceDir(assemblyName.Name);
                }
                if (string.IsNullOrWhiteSpace(path) || !File.Exists(path))
                {
                    path = FindDependencyByRootDir(assemblyName.Name);
                }
                if (string.IsNullOrWhiteSpace(path) || !File.Exists(path))
                {
                    return null;
                }
                return AssemblyLoadContext.Default.LoadFromStream(File.OpenRead(path));
            };
            var assembly = AssemblyLoadContext.Default.LoadFromStream(File.OpenRead(assemblyPath));
            return assembly;
        }

        private string FindDependencyByRootDir(string assemblyName)
        {
            var path = Path.Combine(AppDomain.CurrentDomain.BaseDirectory,"Dependencies", assemblyName +".dll");
            if(File.Exists(path))
            {
                return path;
            }
            return string.Empty;
        }

        private string FindDependencyByInternalServiceDir(string assemblyName)
        {
            var dirs = Directory.GetDirectories(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "InternalService"));
            foreach(var dir in dirs)
            {
                var path = Path.Combine(dir, assemblyName +".dll");
                if(File.Exists(path))
                {
                    return path;
                }
            }
            return string.Empty;
        }

        private string FindDependencyByNetServiceDir(string assemblyName)
        {
            var dirs = Directory.GetDirectories(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "NetService"));
            foreach(var dir in dirs)
            {
                var path = Path.Combine(dir, assemblyName +".dll");
                if(File.Exists(path))
                {
                    return path;
                }
            }
            return string.Empty;
        }
    }
}