﻿using WingRpcSDK.JsonRpc.Driver;
using System;
using WingCloudServer.Loader;
using System.IO;

namespace WingCloudServer
{
    internal class InProcessServiceLoader : BaseServiceLoader
    {
        private DllWatcher _dllWatcher;

        /// <summary>
        /// RegisterService
        /// </summary>
        /// <param name="serviceType"></param>
        /// <param name="service"></param>
        internal override void RegisterService(Type serviceType, object service)
        {
            var baseService = (BaseRpcService)service;
            if(baseService!=null)
            {
                baseService.InProcessClient = RpcInProcessClient;
                baseService.Name = string.IsNullOrWhiteSpace(baseService.Name) ? serviceType.Name.Trim('I') : baseService.Name;
                baseService.InitService();
                CallRegister(InProcessServer, serviceType, baseService);
            }
        }

        /// <summary>
        /// Start service
        /// </summary>
        internal override void StartService()
        {
            InProcessServer.Start();
        }

        /// <summary>
        /// Stop service
        /// </summary>
        internal override void StopService()
        {
            InProcessServer.End();
        }

        internal override void LoadModules(string path)
        {
            _dllWatcher = new DllWatcher(path, () => {
                lock(RestartLocker)
                {
                    Console.WriteLine("Server is reboot now...");
                    var startServerBat = string.Empty;
                    switch(Environment.OSVersion.Platform)
                    {
                        case PlatformID.Win32NT:
                            startServerBat = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "start.bat");
                        break;
                        case PlatformID.MacOSX:
                            startServerBat = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "macstart.bat");
                        break;
                        default:
                            startServerBat = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "start.sh");
                        break;
                    }
                    ProcessStarter.StartTool("ServerReboot", $" WingCloudeServer {startServerBat}");
                    Environment.Exit(0);    
                }
            });
            base.LoadModules(path);
        }
    }
}
