﻿using WingRpcSDK.JsonRpc.Driver;
using JsonRpcLite.Network;
using System;
using System.Collections.Concurrent;
using WingCloudServer.Loader;
using System.IO;

namespace WingCloudServer
{
    internal class NetServiceLoader: BaseServiceLoader
    {
        private DllWatcher _dllWatcher;
        private RpcServer _netRpcService;
        private ConcurrentDictionary<string, BaseRpcService> _netRpcServiceDic = new();

        public NetServiceLoader(int port)
        {
            _netRpcService = new(new JsonRpcHttpServerEngine($"http://*:{port}/"));
        }

        /// <summary>
        /// Register service
        /// </summary>
        /// <param name="serviceType">Service type</param>
        /// <param name="service">Service instance</param>
        internal override void RegisterService(Type serviceType, object service)
        {
            var baseService = (BaseRpcService)service;
            if(baseService!=null)
            {
                baseService.InProcessClient = RpcInProcessClient;
                baseService.InitService();
                baseService.Name = string.IsNullOrWhiteSpace(baseService.Name) ? serviceType.Name.Trim('I') : baseService.Name;
                CallRegister(_netRpcService, serviceType, service);
                _netRpcServiceDic.AddOrUpdate(baseService.Name, baseService, (k, v) => baseService);
            }
        }


        /// <summary>
        /// Start service
        /// </summary>
        internal override void StartService()
        {
            _netRpcService?.Start();
        }

        /// <summary>
        /// Stop service
        /// </summary>
        internal override void StopService()
        {
            _netRpcService?.End();
        }

        internal override void LoadModules(string path)
        {
            _dllWatcher = new DllWatcher(path, () => {
                lock (RestartLocker)
                {
                    Console.WriteLine("Server is reboot now...");
                    var startServerBat = string.Empty;
                    switch(Environment.OSVersion.Platform)
                    {
                        case PlatformID.Win32NT:
                            startServerBat = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "start.bat");
                        break;
                        case PlatformID.MacOSX:
                            startServerBat = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "macstart.bat");
                        break;
                        default:
                            startServerBat = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "start.sh");
                        break;
                    }
                    ProcessStarter.StartTool("ServerReboot", startServerBat);
                }
            });
            base.LoadModules(path);
        }
    }
}
