﻿using System;
using System.IO;

using WingRpcSDK.Common;
namespace WingCloudServer
{
    internal class ThreadServiceContainer : IServiceContainer
    {
        private InProcessServiceLoader _inProcessServiceLoader;
        private NetServiceLoader _netServiceLoader;

        public ThreadServiceContainer(int rpcServerPort)
        {
            _inProcessServiceLoader = new();
            _netServiceLoader = new(rpcServerPort);
        }

        string IServiceContainer.ServiceName => "WingCloudServer";

        /// <summary>
        /// Load all dll
        /// </summary>
        void IServiceContainer.LoadDlls()
        {
            _inProcessServiceLoader.LoadModules(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "InternalService"));
            _netServiceLoader.LoadModules(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "NetService"));
        }

        /// <summary>
        /// Start all service
        /// </summary>
        void IServiceContainer.StartService()
        {
            _inProcessServiceLoader.StartService();
            _netServiceLoader.StartService();
        }

        void IServiceContainer.StopService()
        {
            _inProcessServiceLoader.StopService();
            _netServiceLoader.StopService();
        }
    }
}
