using System;
using System.Diagnostics;
using System.IO;

namespace WingCloudServer.Loader
{
    public class ProcessStarter
    {
        private ProcessStarter(){}

        /// <summary>
        /// 开始执行工具程序
        /// </summary>
        /// <param name="processName">工具程序名称</param>
        public static void StartTool(string processName, string paramsValue="")
        {
            var toolsRootPath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Tools");           
            var subPath = GetPlatfromTitle();
            var tool = Path.Combine(toolsRootPath,subPath, $"{subPath}_{processName}");
            if (Process.GetProcessesByName(Path.GetFileName(tool)).Length == 0)
            {                                
                using(Process process = new Process())
                {
                    process.StartInfo.WorkingDirectory = AppDomain.CurrentDomain.BaseDirectory;
                    process.StartInfo.FileName = tool;
                    process.StartInfo.Arguments = paramsValue;
                    process.Start();
                }
            }
        }

        /// <summary>
        /// 停止工具程序
        /// </summary>
        /// <param name="processName">工具程序名称</param>
        public static void StopTool(string processName)
        {
            var proc = Process.GetProcessesByName($"{GetPlatfromTitle()}_{processName}");
            foreach(var p in proc)
            {
                try
                {
                    Console.WriteLine($"close {p.ProcessName}");
                    Process.GetProcessById(p.Id).Kill();
                }
                catch(Exception pex)
                {
                    Console.WriteLine($"{pex}");
                }
            }
        }

        private static string GetPlatfromTitle()
        {
            string platfrom;
            switch(Environment.OSVersion.Platform)
            {
                case PlatformID.Win32NT:
                    platfrom = "win";
                break;
                case PlatformID.MacOSX:
                    platfrom = "mac";
                break;
                default:
                    platfrom = "lin";
                break;
            }
            return platfrom;
        }
    }
}