﻿using WingRpcSDK.Common;
using System;
using System.Text;
using System.Diagnostics;
using System.Threading.Tasks;
using System.IO;
using System.Net;
using System.Collections.Generic;
using WingCloudServer.Loader;
using WingServerCommon.Log;
using WingServerCommon.Config;

namespace WingCloudServer
{
    class Program
    {
        private static IServiceContainer _rpcServiceContainer;
        static void Main(string[] args)
        {
            var logEngine = new ServerLoggerEngine();
            Logger.RegisterEngine(logEngine);
            
            while (!StartServer())
            {
                Console.WriteLine("The server is abnormal,wait 5 second will to reboot server!");
                Task.Delay(5000).Wait();
            }
        }

        private static bool StartServer()
        {
            try
            {
                Logger.WriteLineInfo($"Starting Server");
                
                //启动短信服务
                ProcessStarter.StartTool("Sms.Tool"); //Should be actived by deploy service in the future
                Logger.WriteLineInfo($"Process Sms.Tool started!");
                
                var rpcServerPort = ConfigurationManager.ServerGatewayPort;
                _rpcServiceContainer = new ThreadServiceContainer(rpcServerPort);
                _rpcServiceContainer.StartService();
                _rpcServiceContainer.LoadDlls();

                Logger.WriteLineInfo($"Server is running on port {rpcServerPort}!");

                string readStr = Console.ReadLine();
                while (readStr.ToUpper() != "Q")
                {
                    readStr = Console.ReadLine();
                }
                _rpcServiceContainer.StopService();
                return true;
            }
            catch (Exception ex)
            {
                Logger.WriteLineError($"Applaction exception {ex} exit,reboot now!");
                return false;
            }
        }
    }
}
